@@ -15,7 +15,6 @@ sub new {
     my $self = {};
     bless $self, $class;
 
-    $self->_dir([ @{$top->_curr_comps()} ]);
     $self->_stat_ret($top->_top_stat_copy());
 
     my $find = { %{$from->_inodes()} };
@@ -27,8 +26,6 @@ sub new {
 
     $self->_last_dir_scanned(undef);
 
-    $from->_dir($self->_dir_copy());
-
     $top->_fill_actions($self);
 
     push @{$top->_curr_comps()}, "";
@@ -175,7 +172,7 @@ __PACKAGE__->_make_copy_methods([qw(
 
 use Carp;
 
-our $VERSION = '0.2.0';
+our $VERSION = '0.2.1';
 
 sub new {
     my ($class, $options, @targets) = @_;
@@ -237,6 +234,9 @@ sub _calc_current_item_obj {
         dir_components => \@comps,
         base => shift(@comps),
         stat_ret => scalar($self->_top_stat_copy()),
+        is_file => scalar(-f _),
+        is_dir => scalar(-d _),
+        is_link => $self->_top_is_link(),
     };
 
     if ($self->_curr_not_a_dir())
@@ -497,7 +497,7 @@ sub _check_subdir
 sub _warn_about_loop
 {
     my $self = shift;
-    my $ptr = shift;
+    my $component_idx = shift;
 
     # Don't pass strings directly to the format.
     # Instead - use %s
@@ -505,8 +505,10 @@ sub _warn_about_loop
     warn(
         sprintf(
             "Avoid loop %s => %s\n",
-                $ptr->_dir_as_string(),
-                $self->_curr_path()
+                File::Spec->catdir(
+                    @{$self->_curr_comps()}[0 .. $component_idx]
+                ),
+                $self->_curr_path(),
         )
     );
 
@@ -520,7 +522,7 @@ sub _is_loop {
     my $lookup = $self->_current->_inodes;
 
     if (exists($lookup->{$key})) {
-        $self->_warn_about_loop($self->_dir_stack->[$lookup->{$key}]);
+        $self->_warn_about_loop($lookup->{$key});
         return 1;
     }
     else {
@@ -565,7 +567,9 @@ sub _context_less_eval {
 sub _open_dir {
     my $self = shift;
 
-    return $self->_current()->_component_open_dir();
+    return $self->_current()->_component_open_dir(
+        $self->_curr_path()
+    );
 }
 
 sub set_traverse_to
@@ -589,8 +593,6 @@ sub get_current_node_files_list
 {
     my $self = shift;
 
-    $self->_current->_dir($self->_curr_comps());
-
     # _open_dir can return undef if $self->_current is not a directory.
     if ($self->_open_dir())
     {
